/***************************************************************************\
*****************************************************************************
**                                                                         **
** header file:    memory.h                                                **
** last revision:  16/12/2003                                              **
**                                                                         **
*****************************************************************************
*****************************************************************************
**                                                                         **
** This program is free software; you can redistribute it and/or modify it **
** under the terms of the GNU General Public License as published by the   **
** Free Software Foundation; either version 2, or (at your option) any     **
** later version.                                                          **
**                                                                         **
** This program is distributed in the hope that it will be useful,         **
** but WITHOUT ANY WARRANTY; without even the implied warranty of          **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           **
** GNU General Public License for more details.                            **
**                                                                         **
** You should have received a copy of the GNU General Public License       **
** along with this program; if not, write to the Free Software             **
** Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.                     **
**                                                                         **
*****************************************************************************
\***************************************************************************/

//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------

#ifndef __MEMORY_H__
#define __MEMORY_H__

/** The Structures **/
typedef struct mem_brick_t
{
  void *datas;
  size_t size;
  struct mem_brick_t *next_brick;
  struct mem_brick_t *prev_brick;
} MEM_BRICK;

typedef struct mem_manager_t
{
  MEM_BRICK *head;
  MEM_BRICK *tail;
} MEM_MANAGER ;


/** The Functions **/
MEM_MANAGER *mem_create(void);                                       // This Creates A Memory Manager
MEM_MANAGER *mem_recreate(MEM_MANAGER *mem);                         // This Re-Creates A Memory Manager
int mem_alloc(MEM_MANAGER *mem, void **datas, const size_t size);    // This Allocates Some Memory
int mem_realloc(MEM_MANAGER *mem, void **datas, const size_t size);  // This Reallocates Some Memory
size_t mem_sizeof(MEM_MANAGER *mem, const void *datas);              // This Returns The Amount Of Allocated Memory
int mem_free(MEM_MANAGER *mem, void **datas);                        // This Frees Some Memory
void mem_free_all(MEM_MANAGER *mem);                                 // This Frees All Allocated Memory
void mem_destroy(MEM_MANAGER *mem);                                  // This Destroys A Memory Manager

#endif

//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------
